@@ -116,7 +116,7 @@ public:
     /** Copies another buffer.
 
         This buffer will make its own copy of the other's data, unless the buffer was created
-        using an external data buffer, in which case boths buffers will just point to the same
+        using an external data buffer, in which case both buffers will just point to the same
         shared block of data.
     */
     AudioBuffer (const AudioBuffer& other)
@@ -180,7 +180,7 @@ public:
           size (other.size),
           allocatedBytes (other.allocatedBytes),
           allocatedData (std::move (other.allocatedData)),
-          isClear (other.isClear)
+          isClear (other.isClear.load())
     {
         if (numChannels < (int) numElementsInArray (preallocatedChannelSpace))
         {
@@ -206,7 +206,7 @@ public:
         size = other.size;
         allocatedBytes = other.allocatedBytes;
         allocatedData = std::move (other.allocatedData);
-        isClear = other.isClear;
+        isClear = other.isClear.load();
 
         if (numChannels < (int) numElementsInArray (preallocatedChannelSpace))
         {
@@ -340,7 +340,7 @@ public:
         if (newNumSamples != size || newNumChannels != numChannels)
         {
             auto allocatedSamplesPerChannel = ((size_t) newNumSamples + 3) & ~3u;
-            auto channelListSize = ((sizeof (Type*) * (size_t) (newNumChannels + 1)) + 15) & ~15u;
+            auto channelListSize = ((static_cast<size_t> (1 + newNumChannels) * sizeof (Type*)) + 15) & ~15u;
             auto newTotalBytes = ((size_t) newNumChannels * (size_t) allocatedSamplesPerChannel * sizeof (Type))
                                     + channelListSize + 32;
 
@@ -1071,12 +1071,21 @@ private:
     Type** channels;
     HeapBlock<char, true> allocatedData;
     Type* preallocatedChannelSpace[32];
-    bool isClear = false;
+    std::atomic<bool> isClear { false };
 
     void allocateData()
     {
+        static_assert (std::alignment_of<Type>::value <= std::alignment_of<std::max_align_t>::value,
+                       "AudioBuffer cannot hold types with alignment requirements larger than that guaranteed by malloc");
         jassert (size >= 0);
-        auto channelListSize = sizeof (Type*) * (size_t) (numChannels + 1);
+
+        auto channelListSize = (size_t) (numChannels + 1) * sizeof (Type*);
+        auto requiredSampleAlignment = std::alignment_of<Type>::value;
+        size_t alignmentOverflow = channelListSize % requiredSampleAlignment;
+
+        if (alignmentOverflow != 0)
+            channelListSize += requiredSampleAlignment - alignmentOverflow;
+
         allocatedBytes = (size_t) numChannels * (size_t) size * sizeof (Type) + channelListSize + 32;
         allocatedData.malloc (allocatedBytes);
         channels = reinterpret_cast<Type**> (allocatedData.get());
