@@ -240,7 +240,15 @@ public:
     /** Returns a pointer to the first element in the array.
         This method is provided for compatibility with standard C++ iteration mechanisms.
     */
-    inline ObjectClass** begin() const noexcept
+    inline ObjectClass** begin() noexcept
+    {
+        return values.begin();
+    }
+
+    /** Returns a pointer to the first element in the array.
+        This method is provided for compatibility with standard C++ iteration mechanisms.
+    */
+    inline ObjectClass* const* begin() const noexcept
     {
         return values.begin();
     }
@@ -248,7 +256,15 @@ public:
     /** Returns a pointer to the element which follows the last element in the array.
         This method is provided for compatibility with standard C++ iteration mechanisms.
     */
-    inline ObjectClass** end() const noexcept
+    inline ObjectClass** end() noexcept
+    {
+        return values.end();
+    }
+
+    /** Returns a pointer to the element which follows the last element in the array.
+        This method is provided for compatibility with standard C++ iteration mechanisms.
+    */
+    inline ObjectClass* const* end() const noexcept
     {
         return values.end();
     }
@@ -256,7 +272,15 @@ public:
     /** Returns a pointer to the first element in the array.
         This method is provided for compatibility with the standard C++ containers.
     */
-    inline ObjectClass** data() const noexcept
+    inline ObjectClass** data() noexcept
+    {
+        return begin();
+    }
+
+    /** Returns a pointer to the first element in the array.
+        This method is provided for compatibility with the standard C++ containers.
+    */
+    inline ObjectClass* const* data() const noexcept
     {
         return begin();
     }
@@ -270,8 +294,8 @@ public:
     int indexOf (const ObjectClass* objectToLookFor) const noexcept
     {
         const ScopedLockType lock (getLock());
-        auto** e = values.begin();
-        auto** endPointer = values.end();
+        auto* e = values.begin();
+        auto* endPointer = values.end();
 
         while (e != endPointer)
         {
@@ -299,8 +323,8 @@ public:
     bool contains (const ObjectClass* objectToLookFor) const noexcept
     {
         const ScopedLockType lock (getLock());
-        auto** e = values.begin();
-        auto** endPointer = values.end();
+        auto* e = values.begin();
+        auto* endPointer = values.end();
 
         while (e != endPointer)
         {
@@ -437,8 +461,9 @@ public:
 
             if (indexToChange < values.size())
             {
-                releaseObject (values[indexToChange]);
+                auto* e = values[indexToChange];
                 values[indexToChange] = newObject;
+                releaseObject (e);
             }
             else
             {
@@ -447,6 +472,20 @@ public:
         }
     }
 
+    /** Replaces an object in the array with a different one.
+
+        If the index is less than zero, this method does nothing.
+        If the index is beyond the end of the array, the new object is added to the end of the array.
+
+        The object being added has its reference count increased, and if it's replacing
+        another object, then that one has its reference count decreased, and may be deleted.
+
+        @param indexToChange        the index whose value you want to change
+        @param newObject            the new value to set for this index.
+        @see add, insert, remove
+    */
+    void set (int indexToChange, const ObjectClassPtr& newObject)       { set (indexToChange, newObject.get()); }
+
     /** Adds elements from another array to the end of this array.
 
         @param arrayToAddFrom       the array from which to copy the elements
@@ -570,9 +609,9 @@ public:
 
         if (isPositiveAndBelow (indexToRemove, values.size()))
         {
-            auto** e = values.begin() + indexToRemove;
-            releaseObject (*e);
+            auto* e = *(values.begin() + indexToRemove);
             values.removeElements (indexToRemove, 1);
+            releaseObject (e);
 
             if ((values.size() << 1) < values.capacity())
                 minimiseStorageOverheads();
@@ -595,10 +634,10 @@ public:
 
         if (isPositiveAndBelow (indexToRemove, values.size()))
         {
-            auto** e = values.begin() + indexToRemove;
-            removedItem = *e;
-            releaseObject (*e);
+            auto* e = *(values.begin() + indexToRemove);
+            removedItem = e;
             values.removeElements (indexToRemove, 1);
+            releaseObject (e);
 
             if ((values.size() << 1) < values.capacity())
                 minimiseStorageOverheads();
@@ -656,14 +695,14 @@ public:
 
         if (numberToRemove > 0)
         {
-            for (int i = startIndex; i < endIndex; ++i)
-            {
-                releaseObject (values[i]);
-                values[i] = nullptr; // (in case one of the destructors accesses this array and hits a dangling pointer)
-            }
+            Array<ObjectClass*> objectsToRemove;
+            objectsToRemove.addArray (values.begin() + startIndex, numberToRemove);
 
             values.removeElements (startIndex, numberToRemove);
 
+            for (auto& o : objectsToRemove)
+                releaseObject (o);
+
             if ((values.size() << 1) < values.capacity())
                 minimiseStorageOverheads();
         }
@@ -790,7 +829,7 @@ public:
     */
     template <class ElementComparator>
     void sort (ElementComparator& comparator,
-               bool retainOrderOfEquivalentItems = false) const noexcept
+               bool retainOrderOfEquivalentItems = false) noexcept
     {
         // If you pass in an object with a static compareElements() method, this
         // avoids getting warning messages about the parameter being unused
@@ -848,10 +887,14 @@ private:
 
     void releaseAllObjects()
     {
-        for (auto& v : values)
-            releaseObject (v);
+        auto i = values.size();
 
-        values.clear();
+        while (--i >= 0)
+        {
+            auto* e = values[i];
+            values.removeElements (i, 1);
+            releaseObject (e);
+        }
     }
 
     static void releaseObject (ObjectClass* o)
